@@ -2,9 +2,9 @@ import ctypes
 from copy import deepcopy
 from random import choice
 
+from .consts import MAX_LABEL_COST, SMALL_DIVISOR
 from .path import find_path_for_agents, find_shortest_path, \
                   single_source_shortest_path
-from .consts import MAX_LABEL_COST, SMALL_DIVISOR
 
 
 __all__ = ['UI']
@@ -159,6 +159,7 @@ class Link:
     def get_generalized_cost(self, tau, value_of_time):
         return (
             self.travel_time_by_period[tau]
+            + self.route_choice_cost
             + self.toll / max(SMALL_DIVISOR, value_of_time) * 60
         )
 
@@ -174,7 +175,7 @@ class Link:
     # def increase_period_agent_vol(self, tau, agent_type, v):
     #     self.vol_by_period_by_at[tau][agent_type] += v
 
-    def calculate_td_vdfunction(self):
+    def calculate_td_vdf(self):
         for tau in range(self.demand_period_size):
             self.travel_time_by_period[tau] = (
                 self.vdfperiods[tau].run_bpr(self.flow_vol_by_period[tau])
@@ -352,57 +353,53 @@ class Network:
         agent_id = 1
         agent_no = 0
 
-        for orig in self.zones:
-            for dest in self.zones:
-                for at in range(self._agent_type_size):
-                    for dp in range(self._demand_period_size):
-                        if (at, dp, orig, dest) not in column_pool.keys():
-                                continue
+        for k, cv in column_pool.items():
+            if cv.get_od_volume() <= 0:
+                continue
 
-                        cv = column_pool[(at, dp, orig, dest)]
+            # k= (at, dp, orig, dest)
+            at = k[0]
+            orig = k[2]
+            dest = k[3]
 
-                        if cv.get_od_volume() <= 0:
-                            continue
+            vol = int(cv.get_od_volume()+1)
+            for _ in range(vol):
+                # construct agent using valid record
+                agent = Agent(agent_id,
+                              agent_no,
+                              at,
+                              orig,
+                              dest)
 
-                        vol = int(cv.get_od_volume()+1)
+                # step 1 generate o_node_id and d_node_id randomly
+                # according to o_zone_id and d_zone_id
+                agent.o_node_id = choice(
+                    self.zone_to_nodes_dict[orig]
+                )
+                agent.d_node_id = choice(
+                    self.zone_to_nodes_dict[dest]
+                )
 
-                        for _ in range(vol):
-                            # construct agent using valid record
-                            agent = Agent(agent_id,
-                                          agent_no,
-                                          at,
-                                          orig,
-                                          dest)
+                # step 2 update agent_id and agent_seq_no
+                agent_id += 1
+                agent_no += 1
 
-                            # step 1 generate o_node_id and d_node_id randomly
-                            # according to o_zone_id and d_zone_id
-                            agent.o_node_id = choice(
-                                self.zone_to_nodes_dict[orig]
-                            )
-                            agent.d_node_id = choice(
-                                self.zone_to_nodes_dict[dest]
-                            )
+                # step 3: update the g_simulation_start_time_in_min and
+                # g_simulation_end_time_in_min
+                # if agent.departure_time_in_min < g_simulation_start_time_in_min:
+                #     g_simulation_start_time_in_min = agent.departure_time_in_min
+                # if agent.departure_time_in_min > g_simulation_end_time_in_min:
+                #     g_simulation_end_time_in_min = agent.departure_time_in_min
 
-                            # step 2 update agent_id and agent_seq_no
-                            agent_id += 1
-                            agent_no += 1
+                #step 4: add the agent to the time dependent agent list
+                # departure_time = agent.get_dep_simu_intvl()
+                # if departure_time not in self.agent_td_list_dict.keys():
+                #     self.agent_td_list_dict[departure_time] = []
+                # self.agent_td_list_dict[departure_time].append(
+                #     agent.get_seq_no()
+                # )
 
-                            # step 3: update the g_simulation_start_time_in_min and
-                            # g_simulation_end_time_in_min
-                            # if agent.departure_time_in_min < g_simulation_start_time_in_min:
-                            #     g_simulation_start_time_in_min = agent.departure_time_in_min
-                            # if agent.departure_time_in_min > g_simulation_end_time_in_min:
-                            #     g_simulation_end_time_in_min = agent.departure_time_in_min
-
-                            #step 4: add the agent to the time dependent agent list
-                            # departure_time = agent.get_dep_simu_intvl()
-                            # if departure_time not in self.agent_td_list_dict.keys():
-                            #     self.agent_td_list_dict[departure_time] = []
-                            # self.agent_td_list_dict[departure_time].append(
-                            #     agent.get_seq_no()
-                            # )
-
-                            self.agent_list.append(agent)
+                self.agent_list.append(agent)
 
         # 03/22/21, comment out until departure time is enabled
         # in the future release
@@ -568,8 +565,8 @@ class Network:
 
 class Column:
     """ column is path """
-    def __init__(self, seq_no=-1):
-        self.seq_no = seq_no
+    def __init__(self, id=-1):
+        self.id = id
         self.vol = 0
         self.dist = 0
         self.toll = 0
@@ -588,8 +585,8 @@ class Column:
     def get_node_num(self):
         return len(self.nodes)
 
-    def get_seq_no(self):
-        return self.seq_no
+    def get_id(self):
+        return self.id
 
     def get_distance(self):
         return self.dist
@@ -668,7 +665,7 @@ class ColumnVec:
     def __init__(self):
         self.od_vol = 0
         self.route_fixed = False
-        self.path_node_seq_map = {}
+        self.node_seq_paths = []
 
     def is_route_fixed(self):
         return self.route_fixed
@@ -677,16 +674,22 @@ class ColumnVec:
         return self.od_vol
 
     def get_column_num(self):
-        return len(self.path_node_seq_map)
+        return len(self.node_seq_paths)
 
     def get_columns(self):
-        return self.path_node_seq_map
+        return self.node_seq_paths
 
     def get_column(self, k):
-        return self.path_node_seq_map[k]
+        return self.node_seq_paths[k]
 
-    def add_new_column(self, k, col):
-        self.path_node_seq_map[k] = col
+    def add_new_column(self, col):
+        self.node_seq_paths.append(col)
+
+    def set_volume(self, vol):
+        self.od_vol = vol
+
+    def increase_volume(self, vol):
+        self.od_vol += vol
 
 
 class AgentType:
@@ -909,23 +912,23 @@ class SPNetwork(Network):
 
     # the following five are unique to each SPNetwork
     def get_node_preds(self):
-        return self.node_predecessor
+        return super().get_node_preds()
 
     def get_link_preds(self):
-        return self.link_predecessor
+        return super().get_link_preds()
 
     def get_node_label_costs(self):
-        return self.node_label_cost
+        return super().get_node_label_costs()
 
     def get_link_costs(self):
-        return self.link_cost_array
+        return super().get_link_costs()
 
     def get_queue_next(self):
-        return self.queue_next
+        return super().get_queue_next()
 
     def get_last_thru_node(self):
         """ node no of the first potential centroid """
-        return self.get_node_size()
+        return super().get_last_thru_node()
 
 
 class AccessNetwork(Network):
@@ -1048,46 +1051,46 @@ class AccessNetwork(Network):
         return self.map_id_to_no[node_id]
 
     def get_node_size(self):
-        return self.node_size
+        return super().get_node_size()
 
     def get_link_size(self):
-        return self.link_size
+        return super().get_link_size()
 
     def get_from_node_no_arr(self):
-        return self.from_node_no_array
+        return super().get_from_node_no_arr()
 
     def get_to_node_no_arr(self):
-        return self.to_node_no_array
+        return super().get_to_node_no_arr()
 
     def get_first_links(self):
-        return self.first_link_from
+        return super().get_first_links()
 
     def get_last_links(self):
-        return self.last_link_from
+        return super().get_last_links()
 
     def get_sorted_link_no_arr(self):
-        return self.sorted_link_no_array
+        return super().get_sorted_link_no_arr()
 
     def get_node_preds(self):
-        return self.node_predecessor
+        return super().get_node_preds()
 
     def get_link_preds(self):
-        return self.link_predecessor
+        return super().get_link_preds()
 
     def get_node_label_costs(self):
-        return self.node_label_cost
+        return super().get_node_label_costs()
 
     def get_node_label_cost(self, node_no):
         return self.node_label_cost[node_no]
 
     def get_link_costs(self):
-        return self.link_cost_array
+        return super().get_link_costs()
 
     def get_queue_next(self):
-        return self.queue_next
+        return super().get_queue_next()
 
     def get_allowed_uses(self):
-        return self.allowed_uses
+        return super().get_allowed_uses()
 
     def get_last_thru_node(self):
         """ node no of the first centroid """
